#
# *** WARNING: DO NOT EDIT ***
# This file is generated by utils/GenerateTimeZonePopulations.pl
#
#   ./utils/GenerateTimeZonePopulations.pl > utils/oneoff/timezone_populations.pl
#
# Generated: 2019-02-04T10:47:14
#

our $TimeZonesByPopulation = {
  'America/Rainy_River' => '20110',
  'America/Guayaquil' => '2645000',
  'Asia/Almaty' => '1797000',
  'Asia/Kuching' => '570407',
  'Pacific/Nauru' => '11359',
  'America/Indiana/Knox' => '3549',
  'Pacific/Enderbury' => 0,
  'Europe/Andorra' => '76965',
  'Europe/Budapest' => '1759000',
  'Europe/Belgrade' => '1639000',
  'Pacific/Chatham' => '75509',
  'Pacific/Palau' => '21729',
  'Asia/Omsk' => '1178000',
  'Europe/Oslo' => '658390',
  'Atlantic/Bermuda' => '61349',
  'Pacific/Honolulu' => '350395',
  'Asia/Muscat' => '27216',
  'America/Indiana/Winamac' => '2332',
  'Africa/El_Aaiun' => '217732',
  'Africa/Abidjan' => '4980000',
  'Africa/Maputo' => '1099000',
  'America/Rankin_Inlet' => '2334',
  'America/Manaus' => '2094000',
  'Africa/Tripoli' => '1126000',
  'Europe/Stockholm' => '932917',
  'America/Grand_Turk' => 0,
  'America/Cayenne' => '57614',
  'Asia/Hebron' => '160470',
  'Pacific/Tarawa' => '44429',
  'Europe/Riga' => '696593',
  'Atlantic/South_Georgia' => '30',
  'America/Fortaleza' => '2610000',
  'America/Denver' => '704621',
  'America/Creston' => '7839',
  'Asia/Bangkok' => '5687000',
  'Antarctica/Macquarie' => 0,
  'Asia/Aqtobe' => '387807',
  'America/Juneau' => '32094',
  'Europe/Dublin' => '1280000',
  'Europe/London' => '8674000',
  'Pacific/Funafuti' => '4749',
  'America/Caracas' => '4544000',
  'America/Nassau' => '1109',
  'America/Cuiaba' => '585367',
  'Africa/Kigali' => '745261',
  'Europe/Kiev' => '2901000',
  'Pacific/Noumea' => '100237',
  'Europe/Kaliningrad' => '459560',
  'Pacific/Norfolk' => '244703',
  'Asia/Ashgabat' => '1032000',
  'Europe/Prague' => '1267000',
  'Europe/Tirane' => '421286',
  'Asia/Baku' => '2122000',
  'Atlantic/St_Helena' => '4049',
  'Pacific/Pago_Pago' => '4196',
  'Africa/Ouagadougou' => '2200000',
  'Pacific/Kosrae' => '6616',
  'Asia/Taipei' => '2705000',
  'America/Inuvik' => '3463',
  'Africa/Lome' => '837437',
  'Asia/Krasnoyarsk' => '962466',
  'Asia/Vladivostok' => '583673',
  'America/Godthab' => '17316',
  'America/Miquelon' => '6320',
  'Atlantic/Canary' => 0,
  'Africa/Freetown' => '1056000',
  'Asia/Kolkata' => '4497000',
  'America/Menominee' => '8148',
  'America/Swift_Current' => '15503',
  'Europe/Athens' => '664046',
  'America/Montevideo' => '5133',
  'Asia/Hovd' => '76252',
  'Pacific/Kiritimati' => '6447',
  'Asia/Gaza' => '515556',
  'Africa/Dar_es_Salaam' => '4715000',
  'Asia/Qatar' => '2640000',
  'Asia/Colombo' => '752993',
  'Pacific/Guam' => '164229',
  'America/Nome' => '3841',
  'Pacific/Marquesas' => 0,
  'Atlantic/Cape_Verde' => '546388',
  'Africa/Addis_Ababa' => '3385000',
  'America/Santarem' => '194600',
  'Africa/Ndjamena' => '1092000',
  'Asia/Baghdad' => '8765000',
  'America/Santiago' => '6686000',
  'America/Indiana/Tell_City' => '7316',
  'Pacific/Port_Moresby' => '410954',
  'America/Adak' => '326',
  'America/Vancouver' => '175673',
  'America/Puerto_Rico' => '3660000',
  'America/Santa_Isabel' => '187302',
  'America/St_Johns' => '472',
  'Europe/Vienna' => '1868000',
  'Europe/Helsinki' => '628208',
  'Pacific/Apia' => '36735',
  'America/Argentina/Rio_Gallegos' => '97742',
  'Europe/Moscow' => '12380000',
  'Europe/Samara' => '1171000',
  'Asia/Ulaanbaatar' => '1372000',
  'Pacific/Saipan' => '48220',
  'Asia/Macau' => '462800',
  'Pacific/Gambier' => '2450',
  'Asia/Khandyga' => '6861',
  'Africa/Bissau' => '395954',
  'Europe/Copenhagen' => '601448',
  'Atlantic/Faroe' => '49290',
  'Africa/Windhoek' => '325858',
  'Antarctica/Davis' => '68986',
  'Pacific/Tahiti' => '183645',
  'Africa/Conakry' => '1661000',
  'Indian/Cocos' => '6443',
  'Asia/Riyadh' => '8002000',
  'Pacific/Wake' => '30666',
  'Antarctica/Casey' => '2664',
  'Asia/Brunei' => '428697',
  'Asia/Oral' => '271900',
  'America/Bahia_Banderas' => '83739',
  'America/Atikokan' => '2753',
  'Asia/Yekaterinburg' => '1344000',
  'Asia/Pyongyang' => '3199000',
  'Asia/Qyzylorda' => '727990',
  'America/Martinique' => '384896',
  'Africa/Banjul' => '34828',
  'America/Argentina/Catamarca' => '159139',
  'Europe/Rome' => '2873000',
  'Europe/Istanbul' => '14800000',
  'Asia/Shanghai' => '24150000',
  'Africa/Harare' => '1606000',
  'Indian/Mayotte' => '253045',
  'Asia/Jerusalem' => '865721',
  'America/Yakutat' => '613',
  'America/Tegucigalpa' => '1158000',
  'Asia/Aqtau' => '9977',
  'America/New_York' => '8623000',
  'Pacific/Rarotonga' => '14153',
  'America/Metlakatla' => '1405',
  'America/Costa_Rica' => '4910000',
  'Pacific/Galapagos' => '25124',
  'Antarctica/Rothera' => 0,
  'Asia/Tehran' => '8847000',
  'America/Curacao' => '160539',
  'America/Hermosillo' => '884342',
  'Asia/Kuwait' => '4140000',
  'Asia/Samarkand' => '504423',
  'Indian/Kerguelen' => '92',
  'Asia/Choibalsan' => '38150',
  'America/Thunder_Bay' => '108359',
  'America/Detroit' => '673104',
  'Asia/Singapore' => '5710000',
  'Australia/Perth' => '2039000',
  'Africa/Cairo' => '9500000',
  'Asia/Aden' => '760923',
  'America/Porto_Velho' => '304228',
  'Pacific/Chuuk' => '17624',
  'Australia/Eucla' => 0,
  'America/Campo_Grande' => '853622',
  'Africa/Lusaka' => '1743000',
  'America/Argentina/Cordoba' => '1330000',
  'Africa/Dakar' => '2397000',
  'America/Port-au-Prince' => '987310',
  'America/Cayman' => '61559',
  'America/Mexico_City' => '11506',
  'Asia/Amman' => '1036000',
  'America/Belem' => '1440000',
  'Europe/Gibraltar' => '34571',
  'America/North_Dakota/New_Salem' => '938',
  'America/Sitka' => '8689',
  'Europe/Monaco' => '38695',
  'Asia/Karachi' => '14920000',
  'Africa/Malabo' => '187302',
  'Asia/Dubai' => '2715000',
  'Europe/Lisbon' => '545245',
  'America/Argentina/Jujuy' => '305891',
  'Africa/Kinshasa' => '11860000',
  'Asia/Ho_Chi_Minh' => '8426000',
  'Australia/Currie' => '746',
  'Australia/Hobart' => '209287',
  'Asia/Yakutsk' => '239225',
  'America/Los_Angeles' => 4000000,
  'Indian/Maldives' => '436330',
  'Antarctica/Palmer' => '12279',
  'Pacific/Pohnpei' => 0,
  'Asia/Tbilisi' => '1118000',
  'Asia/Dili' => '222323',
  'Africa/Lubumbashi' => '1794000',
  'Asia/Irkutsk' => '578073',
  'Pacific/Auckland' => '462200',
  'America/El_Salvador' => '6380000',
  'Asia/Ust-Nera' => '70500',
  'Antarctica/DumontDUrville' => 0,
  'Atlantic/Stanley' => '1491',
  'Asia/Dhaka' => '14540000',
  'America/Asuncion' => '525294',
  'Europe/Minsk' => '1975000',
  'Asia/Jayapura' => '315872',
  'Pacific/Pitcairn' => '56',
  'Indian/Reunion' => '876562',
  'Asia/Sakhalin' => '497973',
  'Asia/Kathmandu' => '1442000',
  'Pacific/Efate' => '65829',
  'America/Iqaluit' => '6699',
  'Europe/Zaporozhye' => '748984',
  'Europe/Luxembourg' => '111287',
  'Pacific/Easter' => 0,
  'Europe/Sofia' => '1260000',
  'Pacific/Niue' => '1618',
  'America/Moncton' => '69074',
  'America/Maceio' => '1014000',
  'Pacific/Wallis' => '10071',
  'America/Lima' => '37149',
  'Pacific/Fakaofo' => 0,
  'Asia/Makassar' => '1339000',
  'America/Cambridge_Bay' => '1608',
  'Asia/Kashgar' => 0,
  'America/Cancun' => '722800',
  'Asia/Magadan' => '100020',
  'Africa/Douala' => '2447000',
  'America/Phoenix' => '1626000',
  'Indian/Comoro' => '350702',
  'America/Whitehorse' => '9494',
  'Europe/Bucharest' => '2106000',
  'America/Yellowknife' => '17863',
  'Asia/Pontianak' => '573751',
  'Asia/Novosibirsk' => 0,
  'America/Rio_Branco' => '370550',
  'America/Argentina/Ushuaia' => '56956',
  'America/Argentina/Mendoza' => '111900',
  'Australia/Lindeman' => 0,
  'America/Indiana/Indianapolis' => '863002',
  'America/Kentucky/Monticello' => '6112',
  'America/Indiana/Petersburg' => '2307',
  'Australia/Broken_Hill' => '18854',
  'Asia/Novokuznetsk' => '562402',
  'Antarctica/Mawson' => 0,
  'Africa/Maseru' => '227880',
  'Europe/Simferopol' => '332317',
  'Asia/Tashkent' => '2310000',
  'America/Argentina/La_Rioja' => '180995',
  'Indian/Mauritius' => '1270000',
  'Africa/Libreville' => '703904',
  'America/Argentina/San_Juan' => '471389',
  'Europe/Volgograd' => '1016000',
  'Europe/Paris' => '2206000',
  'Atlantic/Reykjavik' => '124800',
  'Africa/Algiers' => '2240000',
  'America/Glace_Bay' => '31597',
  'America/Pangnirtung' => '1481',
  'Europe/Vilnius' => '532762',
  'Asia/Bishkek' => '937400',
  'Africa/Khartoum' => '1411000',
  'Africa/Luanda' => '2487000',
  'America/Boa_Vista' => '326419',
  'Australia/Melbourne' => '4530000',
  'Europe/Chisinau' => '492894',
  'Africa/Lagos' => '21320000',
  'Africa/Sao_Tome' => '71868',
  'Indian/Mahe' => '41816',
  'America/Scoresbysund' => '381',
  'Asia/Kuala_Lumpur' => '1768000',
  'Africa/Mbabane' => '94874',
  'Africa/Djibouti' => '956985',
  'Africa/Bamako' => '1809000',
  'Asia/Tokyo' => '13620000',
  'America/Sao_Paulo' => '12040000',
  'America/Merida' => '892363',
  'Australia/Darwin' => '120652',
  'Atlantic/Azores' => '247494',
  'Africa/Nouakchott' => '958399',
  'America/Jamaica' => '2890000',
  'Atlantic/Madeira' => '245806',
  'America/Halifax' => '7855',
  'America/Montreal' => '760',
  'Indian/Christmas' => 0,
  'America/Danmarkshavn' => 0,
  'Asia/Anadyr' => '11193',
  'America/Matamoros' => '520367',
  'America/Goose_Bay' => 0,
  'America/Barbados' => '285719',
  'Pacific/Midway' => '5093',
  'Africa/Casablanca' => '4370000',
  'Africa/Bangui' => '734350',
  'Europe/Tallinn' => '450305',
  'America/Anchorage' => '294356',
  'Africa/Blantyre' => '1069000',
  'America/Santo_Domingo' => '1112000',
  'Africa/Mogadishu' => '2265000',
  'America/Indiana/Marengo' => '814',
  'America/Dawson' => '4157',
  'Asia/Nicosia' => '330000',
  'America/La_Paz' => '551',
  'Africa/Gaborone' => '231626',
  'America/Ojinaga' => '28040',
  'Europe/Warsaw' => '1754000',
  'America/Eirunepe' => '20742',
  'Africa/Ceuta' => '84519',
  'Africa/Niamey' => '1027000',
  'America/Nipigon' => '1631',
  'Asia/Dushanbe' => '780000',
  'Europe/Madrid' => '3166000',
  'Europe/Amsterdam' => '851573',
  'Antarctica/Syowa' => 0,
  'Australia/Sydney' => '4921000',
  'America/Araguaina' => '109571',
  'Asia/Kabul' => '3678000',
  'America/Chihuahua' => '878062',
  'Australia/Adelaide' => '1317000',
  'Indian/Chagos' => 0,
  'America/Argentina/San_Luis' => '250947',
  'America/Monterrey' => '1109000',
  'Asia/Damascus' => '1711000',
  'America/Guyana' => '777859',
  'America/Edmonton' => '1585',
  'Europe/Malta' => '430835',
  'America/Indiana/Vevay' => '1652',
  'America/Toronto' => '4814',
  'Asia/Beirut' => '2006000',
  'Europe/Berlin' => '3520000',
  'Asia/Yerevan' => '1060000',
  'America/Argentina/Salta' => '535303',
  'America/Mazatlan' => '438434',
  'Asia/Vientiane' => '196731',
  'Asia/Chongqing' => '3967000',
  'Asia/Phnom_Penh' => '1502000',
  'Asia/Kamchatka' => '322079',
  'America/Argentina/Tucuman' => '549163',
  'Africa/Monrovia' => '1011000',
  'America/Chicago' => '2716000',
  'America/North_Dakota/Center' => 0,
  'Africa/Nairobi' => 4000000,
  'Pacific/Tongatapu' => '75416',
  'America/Belize' => '374681',
  'America/Noronha' => 0,
  'Pacific/Majuro' => '21019',
  'America/Boise' => '226570',
  'Asia/Urumqi' => '2412000',
  'America/Indiana/Vincennes' => '17729',
  'America/Paramaribo' => '240924',
  'America/Winnipeg' => '663617',
  'America/Managua' => '1043000',
  'America/Blanc-Sablon' => 0,
  'America/Dawson_Creek' => '12978',
  'America/Bogota' => '8567',
  'Africa/Accra' => '2270000',
  'Indian/Antananarivo' => '2610000',
  'America/Panama' => '4100000',
  'Australia/Brisbane' => '2309000',
  'Asia/Harbin' => '5015000',
  'America/Argentina/Buenos_Aires' => '2890000',
  'Asia/Seoul' => '9971000',
  'Europe/Zurich' => '372857',
  'Pacific/Guadalcanal' => '109382',
  'Pacific/Kwajalein' => '11408',
  'Africa/Porto-Novo' => '267191',
  'Africa/Brazzaville' => '1827000',
  'Africa/Bujumbura' => '330504',
  'Asia/Rangoon' => '7611000',
  'Australia/Lord_Howe' => '2367',
  'Asia/Jakarta' => '10190000',
  'Africa/Asmara' => '804000',
  'America/Bahia' => '15200000',
  'America/North_Dakota/Beulah' => '3266',
  'America/Kentucky/Louisville' => '621349',
  'Asia/Bahrain' => '1490000',
  'America/Havana' => '3076',
  'America/Tijuana' => '1697000',
  'Africa/Kampala' => '1507000',
  'America/Port_of_Spain' => '49657',
  'America/Regina' => '105',
  'America/Antigua' => '102012',
  'America/Resolute' => '198',
  'Africa/Tunis' => '2256000',
  'America/Thule' => 0,
  'Europe/Uzhgorod' => '115163',
  'Asia/Thimphu' => '104000',
  'Asia/Manila' => '1780000',
  'Europe/Brussels' => '1114000',
  'Antarctica/Vostok' => '3488',
  'America/Guatemala' => '16900000',
  'America/Recife' => '1555000',
  'Africa/Johannesburg' => '4435000',
  'Pacific/Fiji' => '905502',
  'Asia/Hong_Kong' => '7235000'
}
;